@@ -25,8 +25,7 @@ def bonjour_cli():
 @click.option('--timeout', default=DEFAULT_BROWSE_TIMEOUT, type=click.INT)
 @click.option('--pair-records', type=click.Path(dir_okay=True, file_okay=False, exists=True),
               help='pair records to attempt validation with')
-@click.option('--color/--no-color', default=True)
-def cli_browse(timeout: int, pair_records: str, color: bool):
+def cli_browse(timeout: int, pair_records: str):
     """ browse devices over bonjour """
     records = []
     if pair_records is not None:
@@ -38,4 +37,4 @@ def cli_browse(timeout: int, pair_records: str, color: bool):
         device = device.asdict()
         output.append(device)
 
-    print_json(output, colored=color)
+    print_json(output)
