@@ -19,8 +19,8 @@ def cli():
 @click.argument('name')
 @click.argument('timeout', type=click.INT)
 @click.argument('details', required=False)
-def power_assertion(lockdown: LockdownClient, type, name, timeout, details):
+def power_assertion(service_provider: LockdownClient, type, name, timeout, details):
     """ Create a power assertion (wraps IOPMAssertionCreateWithName()) """
-    with PowerAssertionService(lockdown).create_power_assertion(type, name, timeout, details):
+    with PowerAssertionService(service_provider).create_power_assertion(type, name, timeout, details):
         print('> Hit Ctrl+C to exit')
         time.sleep(timeout)
