@@ -1,9 +1,10 @@
 import asyncio
 import logging
+import re
 from abc import ABC, abstractmethod
 from contextlib import asynccontextmanager
 from functools import update_wrapper
-from typing import Optional, Type
+from typing import Iterable, List, Optional, Type
 
 import click
 import inquirer3
@@ -12,6 +13,7 @@ import uvicorn
 from inquirer3.themes import GreenPassion
 from prompt_toolkit import HTML, PromptSession
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
+from prompt_toolkit.completion.base import CompleteEvent, Completer, Completion, Document
 from prompt_toolkit.history import FileHistory
 from prompt_toolkit.lexers import PygmentsLexer
 from prompt_toolkit.patch_stdout import patch_stdout
@@ -29,6 +31,34 @@ from pymobiledevice3.services.web_protocol.driver import By, Cookie, WebDriver
 from pymobiledevice3.services.web_protocol.inspector_session import InspectorSession
 from pymobiledevice3.services.webinspector import SAFARI, Page, WebinspectorService
 
+SCRIPT = '''
+function inspectedPage_evalResult_getCompletions(primitiveType) {{
+    var resultSet={{}};
+    var object = primitiveType;
+    for(var o=object;o;o=o.__proto__) {{
+
+        try{{
+            var names=Object.getOwnPropertyNames(o);
+            for(var i=0;i<names.length;++i)
+                resultSet[names[i]]=true;
+        }} catch(e){{}}
+    }}
+    return resultSet;
+}}
+
+try {{
+    inspectedPage_evalResult_getCompletions({object})
+}} catch (e) {{}}
+'''
+
+JS_RESERVED_WORDS = ['abstract', 'arguments', 'await', 'boolean', 'break', 'byte', 'case', 'catch', 'char', 'class',
+                     'const', 'continue', 'debugger', 'default', 'delete', 'do', 'double', 'else', 'enum', 'eval',
+                     'export', 'extends', 'false', 'final', 'finally', 'float', 'for', 'function', 'goto', 'if',
+                     'implements', 'import', 'in', 'instanceof', 'int', 'interface', 'let', 'long', 'native', 'new',
+                     'null', 'package', 'private', 'protected', 'public', 'return', 'short', 'static', 'super',
+                     'switch', 'synchronized', 'this', 'throw', 'throws', 'transient', 'true', 'try', 'typeof', 'var',
+                     'void', 'volatile', 'while', 'with', 'yield', ]
+
 logger = logging.getLogger(__name__)
 
 
@@ -75,7 +105,7 @@ def create_webinspector_and_launch_app(lockdown: LockdownClient, timeout: float,
 @click.option('-v', '--verbose', is_flag=True)
 @click.option('-t', '--timeout', default=3, show_default=True, type=float)
 @catch_errors
-def opened_tabs(lockdown: LockdownClient, verbose, timeout):
+def opened_tabs(service_provider: LockdownClient, verbose, timeout):
     """
     Show all currently opened tabs.
 
@@ -83,7 +113,7 @@ def opened_tabs(lockdown: LockdownClient, verbose, timeout):
     Opt-in:
         Settings -> Safari -> Advanced -> Web Inspector
     """
-    inspector = WebinspectorService(lockdown=lockdown, loop=asyncio.get_event_loop())
+    inspector = WebinspectorService(lockdown=service_provider, loop=asyncio.get_event_loop())
     inspector.connect(timeout)
     while not inspector.connected_application:
         inspector.flush_input()
@@ -107,7 +137,7 @@ def opened_tabs(lockdown: LockdownClient, verbose, timeout):
 @click.argument('url')
 @click.option('-t', '--timeout', default=3, show_default=True, type=float)
 @catch_errors
-def launch(lockdown: LockdownClient, url, timeout):
+def launch(service_provider: LockdownClient, url, timeout):
     """
     Launch a specific URL in Safari.
 
@@ -116,7 +146,7 @@ def launch(lockdown: LockdownClient, url, timeout):
         Settings -> Safari -> Advanced -> Web Inspector
         Settings -> Safari -> Advanced -> Remote Automation
     """
-    inspector, safari = create_webinspector_and_launch_app(lockdown, timeout, SAFARI)
+    inspector, safari = create_webinspector_and_launch_app(service_provider, timeout, SAFARI)
     session = inspector.automation_session(safari)
     driver = WebDriver(session)
     print('Starting session')
@@ -152,7 +182,7 @@ driver.add_cookie(
 @webinspector.command(cls=Command)
 @click.option('-t', '--timeout', default=3, show_default=True, type=float)
 @catch_errors
-def shell(lockdown: LockdownClient, timeout):
+def shell(service_provider: LockdownClient, timeout):
     """
     Create an IPython shell for interacting with a WebView.
 
@@ -161,7 +191,7 @@ def shell(lockdown: LockdownClient, timeout):
         Settings -> Safari -> Advanced -> Web Inspector
         Settings -> Safari -> Advanced -> Remote Automation
     """
-    inspector, safari = create_webinspector_and_launch_app(lockdown, timeout, SAFARI)
+    inspector, safari = create_webinspector_and_launch_app(service_provider, timeout, SAFARI)
     session = inspector.automation_session(safari)
     driver = WebDriver(session)
     try:
@@ -182,7 +212,7 @@ def shell(lockdown: LockdownClient, timeout):
 @click.option('--automation', is_flag=True, help='Use remote automation')
 @click.argument('url', required=False, default='')
 @catch_errors
-def js_shell(lockdown: LockdownClient, timeout, automation, url):
+def js_shell(service_provider: LockdownClient, timeout, automation, url):
     """
     Create a javascript shell. This interpreter runs on your local machine,
     but evaluates each expression on the remote
@@ -197,7 +227,7 @@ def js_shell(lockdown: LockdownClient, timeout, automation, url):
     """
 
     js_shell_class = AutomationJsShell if automation else InspectorJsShell
-    asyncio.run(run_js_shell(js_shell_class, lockdown, timeout, url))
+    asyncio.run(run_js_shell(js_shell_class, service_provider, timeout, url))
 
 
 udid = ''
@@ -212,7 +242,7 @@ def create_app():
 @webinspector.command(cls=Command)
 @click.option('--host', default='127.0.0.1')
 @click.option('--port', type=click.INT, default=9222)
-def cdp(lockdown: LockdownClient, host, port):
+def cdp(service_provider: LockdownClient, host, port):
     """
     Start a CDP server for debugging WebViews.
 
@@ -221,18 +251,57 @@ def cdp(lockdown: LockdownClient, host, port):
         chrome://inspect/#devices
     """
     global udid
-    udid = lockdown.udid
+    udid = service_provider.udid
     uvicorn.run('pymobiledevice3.cli.webinspector:create_app', host=host, port=port, factory=True,
                 ws_ping_timeout=None, ws='wsproto', loop='asyncio')
 
 
+def get_js_completions(jsshell: 'JsShell', obj: str, prefix: str) -> List[Completion]:
+    if obj in JS_RESERVED_WORDS:
+        return []
+
+    completions = []
+    try:
+        for key in asyncio.get_running_loop().run_until_complete(
+                jsshell.evaluate_expression(SCRIPT.format(object=obj), return_by_value=True)):
+            if not key.startswith(prefix):
+                continue
+            completions.append(Completion(key.removeprefix(prefix), display=key))
+    except Exception:
+        # ignore every possible exception
+        pass
+    return completions
+
+
+class JsShellCompleter(Completer):
+    def __init__(self, jsshell: 'JsShell'):
+        self.jsshell = jsshell
+
+    def get_completions(
+            self, document: Document, complete_event: CompleteEvent
+    ) -> Iterable[Completion]:
+        text = f'globalThis.{document.text_before_cursor}'
+        text = re.findall('[a-zA-Z_][a-zA-Z_0-9.]+', text)
+        if len(text) == 0:
+            return []
+        text = text[-1]
+        if '.' in text:
+            js_obj, prefix = text.rsplit('.', 1)
+        else:
+            js_obj = text
+            prefix = ''
+
+        return get_js_completions(self.jsshell, js_obj, prefix)
+
+
 class JsShell(ABC):
     def __init__(self):
         super().__init__()
         self.prompt_session = PromptSession(lexer=PygmentsLexer(lexers.JavascriptLexer),
                                             auto_suggest=AutoSuggestFromHistory(),
                                             style=style_from_pygments_cls(get_style_by_name('stata-dark')),
-                                            history=FileHistory(self.webinspector_history_path()))
+                                            history=FileHistory(self.webinspector_history_path()),
+                                            completer=JsShellCompleter(self))
 
     @classmethod
     @abstractmethod
@@ -240,7 +309,7 @@ class JsShell(ABC):
         pass
 
     @abstractmethod
-    async def evaluate_expression(self, exp):
+    async def evaluate_expression(self, exp, return_by_value: bool = False):
         pass
 
     @abstractmethod
@@ -267,7 +336,9 @@ class JsShell(ABC):
                 await self.js_iter()
             except WirError as e:
                 logger.error(e)
-            except (KeyboardInterrupt, InspectorEvaluateError):  # KeyboardInterrupt Control-C
+            except InspectorEvaluateError as e:
+                logger.error(e)
+            except KeyboardInterrupt:  # KeyboardInterrupt Control-C
                 pass
             except EOFError:  # Control-D
                 return
@@ -295,7 +366,7 @@ class AutomationJsShell(JsShell):
             automation_session.stop_session()
             inspector.close()
 
-    async def evaluate_expression(self, exp: str):
+    async def evaluate_expression(self, exp: str, return_by_value: bool = False):
         return self.driver.execute_script(f'return {exp}')
 
     async def navigate(self, url: str):
@@ -316,14 +387,16 @@ class InspectorJsShell(JsShell):
             raise click.exceptions.Exit()
 
         inspector_session = await inspector.inspector_session(application, page)
+        await inspector_session.console_enable()
         await inspector_session.runtime_enable()
+
         try:
             yield cls(inspector_session)
         finally:
             inspector.close()
 
-    async def evaluate_expression(self, exp: str):
-        return await self.inspector_session.runtime_evaluate(exp)
+    async def evaluate_expression(self, exp: str, return_by_value: bool = False):
+        return await self.inspector_session.runtime_evaluate(exp, return_by_value=return_by_value)
 
     async def navigate(self, url: str):
         await self.inspector_session.navigate_to_url(url)
