@@ -108,6 +108,14 @@ IRECV_DEVICES = (
                 display_name='iPhone 14 Pro'),
     IRecvDevice(product_type='iPhone15,3', hardware_model='d74ap', board_id=0x0E, chip_id=0x8120,
                 display_name='iPhone 14 Pro Max'),
+    IRecvDevice(product_type='iPhone15,4', hardware_model='d37ap', board_id=0x08, chip_id=0x8120,
+                display_name='iPhone 15'),
+    IRecvDevice(product_type='iPhone15,5', hardware_model='d38ap', board_id=0x0A, chip_id=0x8120,
+                display_name='iPhone 15 Plus'),
+    IRecvDevice(product_type='iPhone16,1', hardware_model='d83ap', board_id=0x04, chip_id=0x8130,
+                display_name='iPhone 15 Pro'),
+    IRecvDevice(product_type='iPhone16,2', hardware_model='d84ap', board_id=0x06, chip_id=0x8130,
+                display_name='iPhone 15 Pro Max'),
     # iPod
     IRecvDevice(product_type='iPod1,1', hardware_model='n45ap', board_id=0x02, chip_id=0x8900,
                 display_name='iPod Touch (1st gen)'),
