@@ -10,7 +10,7 @@ from contextlib import contextmanager, suppress
 from enum import Enum
 from functools import wraps
 from pathlib import Path
-from typing import Dict, Mapping
+from typing import Dict, Mapping, Optional
 
 from packaging.version import Version
 
@@ -21,46 +21,13 @@ from pymobiledevice3.exceptions import CannotStopSessionError, ConnectionTermina
     MissingValueError, NotPairedError, PairingDialogResponsePendingError, PairingError, PasswordRequiredError, \
     SetProhibitedError, StartServiceError, UserDeniedPairingError
 from pymobiledevice3.irecv_devices import IRECV_DEVICES
+from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
 from pymobiledevice3.pair_records import create_pairing_records_cache_folder, generate_host_id, \
     get_preferred_pair_record
-from pymobiledevice3.service_connection import ServiceConnection
+from pymobiledevice3.service_connection import LockdownServiceConnection
 from pymobiledevice3.usbmux import PlistMuxConnection
-from pymobiledevice3.utils import sanitize_ios_version
 
 SYSTEM_BUID = '30142955-444094379208051516'
-DOMAINS = ['com.apple.disk_usage',
-           'com.apple.disk_usage.factory',
-           'com.apple.mobile.battery',
-           # FIXME: For some reason lockdownd segfaults on this, works sometimes tho
-           # 'com.apple.mobile.debug',
-           'com.apple.iqagent',
-           'com.apple.purplebuddy',
-           'com.apple.PurpleBuddy',
-           'com.apple.mobile.chaperone',
-           'com.apple.mobile.third_party_termination',
-           'com.apple.mobile.lockdownd',
-           'com.apple.mobile.lockdown_cache',
-           'com.apple.xcode.developerdomain',
-           'com.apple.international',
-           'com.apple.mobile.data_sync',
-           'com.apple.mobile.tethered_sync',
-           'com.apple.mobile.mobile_application_usage',
-           'com.apple.mobile.backup',
-           'com.apple.mobile.nikita',
-           'com.apple.mobile.restriction',
-           'com.apple.mobile.user_preferences',
-           'com.apple.mobile.sync_data_class',
-           'com.apple.mobile.software_behavior',
-           'com.apple.mobile.iTunes.SQLMusicLibraryPostProcessCommands',
-           'com.apple.mobile.iTunes.accessories',
-           'com.apple.mobile.internal',  # < iOS 4.0+
-           'com.apple.mobile.wireless_lockdown',  # < iOS 4.0+
-           'com.apple.fairplay',
-           'com.apple.iTunes',
-           'com.apple.mobile.iTunes.store',
-           'com.apple.mobile.iTunes',
-           'com.apple.fmip',
-           'com.apple.Accessibility', ]
 
 DEFAULT_LABEL = 'pymobiledevice3'
 SERVICE_PORT = 62078
@@ -86,7 +53,7 @@ def _reconnect_on_remote_close(f):
         try:
             return f(*args, **kwargs)
         except (BrokenPipeError, ConnectionTerminatedError):
-            self = args[0]
+            self: LockdownClient = args[0]
 
             # first we release the socket on our end to avoid a ResourceWarning
             self.close()
@@ -100,10 +67,10 @@ def _reconnect_on_remote_close(f):
     return _inner_reconnect_on_remote_close
 
 
-class LockdownClient(ABC):
-    def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None, label: str = DEFAULT_LABEL,
-                 system_buid: str = SYSTEM_BUID, pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
-                 port: int = SERVICE_PORT):
+class LockdownClient(ABC, LockdownServiceProvider):
+    def __init__(self, service: LockdownServiceConnection, host_id: str, identifier: str = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
         """
         Create a LockdownClient instance
 
@@ -116,6 +83,7 @@ class LockdownClient(ABC):
         :param pairing_records_cache_folder: Use the following location to search and save pair records
         :param port: lockdownd service port
         """
+        super().__init__()
         self.logger = logging.getLogger(__name__)
         self.service = service
         self.identifier = identifier
@@ -135,11 +103,10 @@ class LockdownClient(ABC):
         self.udid = self.all_values.get('UniqueDeviceID')
         self.unique_chip_id = self.all_values.get('UniqueChipID')
         self.device_public_key = self.all_values.get('DevicePublicKey')
-        self.product_version = self.all_values.get('ProductVersion')
         self.product_type = self.all_values.get('ProductType')
 
     @classmethod
-    def create(cls, service: ServiceConnection, identifier: str = None, system_buid: str = SYSTEM_BUID,
+    def create(cls, service: LockdownServiceConnection, identifier: str = None, system_buid: str = SYSTEM_BUID,
                label: str = DEFAULT_LABEL, autopair: bool = True, pair_timeout: int = None, local_hostname: str = None,
                pair_record: Mapping = None, pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
                **cls_specific_args):
@@ -180,6 +147,10 @@ class LockdownClient(ABC):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
+    @property
+    def product_version(self) -> str:
+        return self.all_values.get('ProductVersion')
+
     @property
     def device_class(self) -> DeviceClass:
         try:
@@ -191,15 +162,6 @@ class LockdownClient(ABC):
     def wifi_mac_address(self) -> str:
         return self.all_values.get('WiFiAddress')
 
-    @property
-    def all_domains(self) -> Mapping:
-        result = self.all_values
-
-        for domain in DOMAINS:
-            result.update({domain: self.get_value(domain)})
-
-        return result
-
     @property
     def short_info(self) -> Dict:
         keys_to_copy = ['DeviceClass', 'DeviceName', 'BuildVersion', 'ProductVersion', 'ProductType']
@@ -214,6 +176,16 @@ class LockdownClient(ABC):
     def share_iphone_analytics_enabled(self) -> bool:
         return self.get_value('com.apple.MobileDeviceCrashCopy', 'ShouldSubmit')
 
+    @property
+    def assistive_touch(self) -> bool:
+        """AssistiveTouch (the on-screen software home button)"""
+        return bool(self.get_value('com.apple.Accessibility').get('AssistiveTouchEnabledByiTunes', 0))
+
+    @assistive_touch.setter
+    def assistive_touch(self, value: bool) -> None:
+        """AssistiveTouch (the on-screen software home button)"""
+        self.set_value(int(value), 'com.apple.Accessibility', 'AssistiveTouchEnabledByiTunes')
+
     @property
     def voice_over(self) -> bool:
         return bool(self.get_value('com.apple.Accessibility').get('VoiceOverTouchEnabledByiTunes', 0))
@@ -258,10 +230,6 @@ class LockdownClient(ABC):
     def preflight_info(self) -> Mapping:
         return self.get_value(key='FirmwarePreflightInfo')
 
-    @property
-    def sanitized_ios_version(self) -> str:
-        return sanitize_ios_version(self.product_version)
-
     @property
     def display_name(self) -> str:
         for irecv_device in IRECV_DEVICES:
@@ -299,6 +267,12 @@ class LockdownClient(ABC):
     def set_locale(self, locale: str) -> None:
         self.set_value(locale, key='Locale', domain='com.apple.international')
 
+    def set_timezone(self, timezone: str) -> None:
+        self.set_value(timezone, key='TimeZone')
+
+    def set_uses24hClock(self, value: bool) -> None:
+        self.set_value(value, key='Uses24HourClock')
+
     @_reconnect_on_remote_close
     def enter_recovery(self):
         return self._request('EnterRecovery')
@@ -429,13 +403,13 @@ class LockdownClient(ABC):
         options['Value'] = value
         return self._request('SetValue', options)
 
-    def get_service_connection_attributes(self, name, escrow_bag=None) -> Mapping:
+    def get_service_connection_attributes(self, name: str, include_escrow_bag: bool = False) -> Mapping:
         if not self.paired:
             raise NotPairedError()
 
         options = {'Service': name}
-        if escrow_bag is not None:
-            options['EscrowBag'] = escrow_bag
+        if include_escrow_bag:
+            options['EscrowBag'] = self.pair_record['EscrowBag']
 
         response = self._request('StartService', options)
         if not response or response.get('Error'):
@@ -446,8 +420,8 @@ class LockdownClient(ABC):
         return response
 
     @_reconnect_on_remote_close
-    def start_service(self, name: str, escrow_bag=None) -> ServiceConnection:
-        attr = self.get_service_connection_attributes(name, escrow_bag=escrow_bag)
+    def start_lockdown_service(self, name: str, include_escrow_bag: bool = False) -> LockdownServiceConnection:
+        attr = self.get_service_connection_attributes(name, include_escrow_bag=include_escrow_bag)
         service_connection = self._create_service_connection(attr['Port'])
 
         if attr.get('EnableServiceSSL', False):
@@ -455,8 +429,9 @@ class LockdownClient(ABC):
                 service_connection.ssl_start(f)
         return service_connection
 
-    async def aio_start_service(self, name: str, escrow_bag=None) -> ServiceConnection:
-        attr = self.get_service_connection_attributes(name, escrow_bag=escrow_bag)
+    async def aio_start_lockdown_service(
+            self, name: str, include_escrow_bag: bool = False) -> LockdownServiceConnection:
+        attr = self.get_service_connection_attributes(name, include_escrow_bag=include_escrow_bag)
         service_connection = self._create_service_connection(attr['Port'])
 
         if attr.get('EnableServiceSSL', False):
@@ -464,16 +439,6 @@ class LockdownClient(ABC):
                 await service_connection.aio_ssl_start(f)
         return service_connection
 
-    def start_developer_service(self, name, escrow_bag=None) -> ServiceConnection:
-        try:
-            return self.start_service(name, escrow_bag)
-        except StartServiceError:
-            self.logger.error(
-                'Failed to connect to required service. Make sure DeveloperDiskImage.dmg has been mounted. '
-                'You can do so using: pymobiledevice3 mounter mount'
-            )
-            raise
-
     def close(self) -> None:
         self.service.close()
 
@@ -507,7 +472,7 @@ class LockdownClient(ABC):
             raise FatalPairingError()
 
     @abstractmethod
-    def _create_service_connection(self, port: int) -> ServiceConnection:
+    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
         """ Used to establish a new ServiceConnection to a given port """
         pass
 
@@ -563,14 +528,29 @@ class LockdownClient(ABC):
 
 
 class UsbmuxLockdownClient(LockdownClient):
+    def __init__(self, service: LockdownServiceConnection, host_id: str, identifier: str = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
+                 usbmux_address: Optional[str] = None):
+        super().__init__(service, host_id, identifier, label, system_buid, pair_record, pairing_records_cache_folder,
+                         port)
+        self.usbmux_address = usbmux_address
+
     @property
     def short_info(self) -> Dict:
         short_info = super().short_info
         short_info['ConnectionType'] = self.service.mux_device.connection_type
         return short_info
 
-    def _create_service_connection(self, port: int) -> ServiceConnection:
-        return ServiceConnection.create_using_usbmux(self.identifier, port, self.service.mux_device.connection_type)
+    def fetch_pair_record(self) -> None:
+        if self.identifier is not None:
+            self.pair_record = get_preferred_pair_record(self.identifier, self.pairing_records_cache_folder,
+                                                         usbmux_address=self.usbmux_address)
+
+    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
+        return LockdownServiceConnection.create_using_usbmux(self.identifier, port,
+                                                             self.service.mux_device.connection_type,
+                                                             usbmux_address=self.usbmux_address)
 
 
 class PlistUsbmuxLockdownClient(UsbmuxLockdownClient):
@@ -582,9 +562,9 @@ class PlistUsbmuxLockdownClient(UsbmuxLockdownClient):
 
 
 class TcpLockdownClient(LockdownClient):
-    def __init__(self, service: ServiceConnection, host_id: str, hostname: str, identifier: str = None,
+    def __init__(self, service: LockdownServiceConnection, host_id: str, hostname: str, identifier: str = None,
                  label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
-                 pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
+                 pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT, keep_alive: bool = True):
         """
         Create a LockdownClient instance
 
@@ -597,19 +577,55 @@ class TcpLockdownClient(LockdownClient):
         :param pair_record: Use this pair record instead of the default behavior (search in host/create our own)
         :param pairing_records_cache_folder: Use the following location to search and save pair records
         :param port: lockdownd service port
+        :param keep_alive: use keep-alive to get notified when the connection is lost
         """
         super().__init__(service, host_id, identifier, label, system_buid, pair_record, pairing_records_cache_folder,
                          port)
+        self._keep_alive = keep_alive
         self.hostname = hostname
 
-    def _create_service_connection(self, port: int) -> ServiceConnection:
-        return ServiceConnection.create_using_tcp(self.hostname, port)
+    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
+        return LockdownServiceConnection.create_using_tcp(self.hostname, port, keep_alive=self._keep_alive)
+
+
+class RemoteLockdownClient(LockdownClient):
+    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
+        raise NotImplementedError(
+            'RemoteXPC service connections should only be created using RemoteServiceDiscoveryService')
+
+    def _handle_autopair(self, *args):
+        # The RemoteXPC version of lockdown doesn't support pairing operations
+        return None
+
+    def pair(self, timeout: int = None) -> None:
+        raise NotImplementedError('RemoteXPC lockdown version does not support pairing operations')
+
+    def unpair(self, timeout: int = None) -> None:
+        raise NotImplementedError('RemoteXPC lockdown version does not support pairing operations')
+
+    def __init__(self, service: LockdownServiceConnection, host_id: str, identifier: str = None,
+                 label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
+                 pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
+        """
+        Create a LockdownClient instance
+
+        :param service: lockdownd connection handler
+        :param host_id: Used as the host identifier for the handshake
+        :param identifier: Used as an identifier to look for the device pair record
+        :param label: lockdownd user-agent
+        :param system_buid: System's unique identifier
+        :param pair_record: Use this pair record instead of the default behavior (search in host/create our own)
+        :param pairing_records_cache_folder: Use the following location to search and save pair records
+        :param port: lockdownd service port
+        """
+        super().__init__(service, host_id, identifier, label, system_buid, pair_record, pairing_records_cache_folder,
+                         port)
 
 
 def create_using_usbmux(serial: str = None, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
                         connection_type: str = None, pair_timeout: int = None, local_hostname: str = None,
                         pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
-                        port: int = SERVICE_PORT) -> UsbmuxLockdownClient:
+                        port: int = SERVICE_PORT, usbmux_address: Optional[str] = None) -> UsbmuxLockdownClient:
     """
     Create a UsbmuxLockdownClient instance
 
@@ -623,11 +639,13 @@ def create_using_usbmux(serial: str = None, identifier: str = None, label: str =
     :param pair_record: Use this pair record instead of the default behavior (search in host/create our own)
     :param pairing_records_cache_folder: Use the following location to search and save pair records
     :param port: lockdownd service port
+    :param usbmux_address: usbmuxd address
     :return: UsbmuxLockdownClient instance
     """
-    service = ServiceConnection.create_using_usbmux(serial, port, connection_type=connection_type)
+    service = LockdownServiceConnection.create_using_usbmux(serial, port, connection_type=connection_type,
+                                                            usbmux_address=usbmux_address)
     cls = UsbmuxLockdownClient
-    with usbmux.create_mux() as client:
+    with usbmux.create_mux(usbmux_address=usbmux_address) as client:
         if isinstance(client, PlistMuxConnection):
             # Only the Plist version of usbmuxd supports this message type
             system_buid = client.get_buid()
@@ -640,12 +658,13 @@ def create_using_usbmux(serial: str = None, identifier: str = None, label: str =
     return cls.create(
         service, identifier=identifier, label=label, system_buid=system_buid, local_hostname=local_hostname,
         pair_record=pair_record, pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout,
-        autopair=autopair)
+        autopair=autopair, usbmux_address=usbmux_address)
 
 
 def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
                      pair_timeout: int = None, local_hostname: str = None, pair_record: Mapping = None,
-                     pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT) -> TcpLockdownClient:
+                     pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
+                     keep_alive: bool = False) -> TcpLockdownClient:
     """
     Create a TcpLockdownClient instance
 
@@ -658,20 +677,21 @@ def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT
     :param pair_record: Use this pair record instead of the default behavior (search in host/create our own)
     :param pairing_records_cache_folder: Use the following location to search and save pair records
     :param port: lockdownd service port
+    :param keep_alive: use keep-alive to get notified when the connection is lost
     :return: TcpLockdownClient instance
     """
-    service = ServiceConnection.create_using_tcp(hostname, port)
+    service = LockdownServiceConnection.create_using_tcp(hostname, port, keep_alive=keep_alive)
     client = TcpLockdownClient.create(
         service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
         pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
-        port=port, hostname=hostname)
+        port=port, hostname=hostname, keep_alive=keep_alive)
     return client
 
 
-def create_using_remote(hostname: str, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
-                        pair_timeout: int = None, local_hostname: str = None, pair_record: Mapping = None,
-                        pairing_records_cache_folder: Path = None,
-                        port: int = SERVICE_PORT) -> TcpLockdownClient:
+def create_using_remote(service: LockdownServiceConnection, identifier: str = None, label: str = DEFAULT_LABEL,
+                        autopair: bool = True, pair_timeout: int = None, local_hostname: str = None,
+                        pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
+                        port: int = SERVICE_PORT) -> RemoteLockdownClient:
     """
     Create a TcpLockdownClient instance over RSD
 
@@ -686,15 +706,8 @@ def create_using_remote(hostname: str, identifier: str = None, label: str = DEFA
     :param port: lockdownd service port
     :return: TcpLockdownClient instance
     """
-    service = ServiceConnection.create_using_tcp(hostname, port)
-    service.send_plist({'Label': label, 'ProtocolVersion': '2', 'Request': 'RSDCheckin'})
-
-    # we expect two responses after the first request
-    service.recv_plist()
-    service.recv_plist()
-
-    client = TcpLockdownClient.create(
+    client = RemoteLockdownClient.create(
         service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
         pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
-        port=port, hostname=hostname)
+        port=port)
     return client
