@@ -1,71 +1,124 @@
 import asyncio
 import base64
 import binascii
+import dataclasses
 import hashlib
 import json
 import logging
 import platform
 import plistlib
+import ssl
+import struct
+import sys
+from abc import ABC, abstractmethod
+from asyncio import CancelledError, StreamReader, StreamWriter
 from collections import namedtuple
-from pathlib import Path
-from ssl import VerifyMode
-from typing import List, Mapping, Optional, TextIO
+from contextlib import asynccontextmanager, suppress
 
-from aioquic.asyncio import QuicConnectionProtocol
-from aioquic.asyncio.client import connect as aioquic_connect
-from aioquic.asyncio.protocol import QuicStreamHandler
-from aioquic.quic.configuration import QuicConfiguration
-from aioquic.quic.connection import QuicConnection
-from aioquic.quic.events import QuicEvent, StreamDataReceived
-from construct import Const, Container, Enum, GreedyBytes, GreedyRange, Int8ul, Int16ub, Int64ul, Prefixed, Struct
+if sys.platform != 'win32':
+    from os import chown
+
+from os import getenv
+from pathlib import Path
+from socket import AF_INET6, create_connection
+from ssl import VerifyMode
+from typing import AsyncGenerator, List, Mapping, Optional, TextIO, cast
+
+import aiofiles
+from construct import Const, Container
+from construct import Enum as ConstructEnum
+from construct import GreedyBytes, GreedyRange, Int8ul, Int16ub, Int64ul, Prefixed, Struct
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives._serialization import Encoding, PublicFormat
+from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
 from cryptography.hazmat.primitives.asymmetric.x25519 import X25519PrivateKey, X25519PublicKey
 from cryptography.hazmat.primitives.ciphers.aead import ChaCha20Poly1305
 from cryptography.hazmat.primitives.kdf.hkdf import HKDF
 from opack import dumps
+
+if sys.platform != 'win32':
+    from pytun_pmd3 import TunTapDevice
+else:
+    from pywintunx_pmd3 import TunTapDevice, set_logger
+
+from qh3.asyncio import QuicConnectionProtocol
+from qh3.asyncio.client import connect as aioquic_connect
+from qh3.asyncio.protocol import QuicStreamHandler
+from qh3.quic import packet_builder
+from qh3.quic.configuration import QuicConfiguration
+from qh3.quic.connection import QuicConnection
+from qh3.quic.events import ConnectionTerminated, DatagramFrameReceived, QuicEvent, StreamDataReceived
 from srptools import SRPClientSession, SRPContext
 from srptools.constants import PRIME_3072, PRIME_3072_GEN
 
-from pymobiledevice3.ca import make_cert
-from pymobiledevice3.pair_records import create_pairing_records_cache_folder, generate_host_id
-from pymobiledevice3.remote.remote_service_discovery import RemoteServiceDiscoveryService
-from pymobiledevice3.remote.xpc_message import XpcInt64Type, XpcUInt64Type
+try:
+    from sslpsk_pmd3.sslpsk import SSLPSKContext
+except ImportError:
+    SSLPSKContext = None
 
-PairingDataComponentType = Enum(Int8ul,
-                                METHOD=0x00,
-                                IDENTIFIER=0x01,
-                                SALT=0x02,
-                                PUBLIC_KEY=0x03,
-                                PROOF=0x04,
-                                ENCRYPTED_DATA=0x05,
-                                STATE=0x06,
-                                ERROR=0x07,
-                                RETRY_DELAY=0x08,
-                                CERTIFICATE=0x09,
-                                SIGNATURE=0x0a,
-                                PERMISSIONS=0x0b,
-                                FRAGMENT_DATA=0x0c,
-                                FRAGMENT_LAST=0x0d,
-                                SESSION_ID=0x0e,
-                                TTL=0x0f,
-                                EXTRA_DATA=0x10,
-                                INFO=0x11,
-                                ACL=0x12,
-                                FLAGS=0x13,
-                                VALIDATION_DATA=0x14,
-                                MFI_AUTH_TOKEN=0x15,
-                                MFI_PRODUCT_TYPE=0x16,
-                                SERIAL_NUMBER=0x17,
-                                MFI_AUTH_TOKEN_UUID=0x18,
-                                APP_FLAGS=0x19,
-                                OWNERSHIP_PROOF=0x1a,
-                                SETUP_CODE_TYPE=0x1b,
-                                PRODUCTION_DATA=0x1c,
-                                APP_INFO=0x1d,
-                                SEPARATOR=0xff)
+from pymobiledevice3.ca import make_cert
+from pymobiledevice3.exceptions import PyMobileDevice3Exception, UserDeniedPairingError
+from pymobiledevice3.pair_records import PAIRING_RECORD_EXT, create_pairing_records_cache_folder, generate_host_id, \
+    get_remote_pairing_record_filename
+from pymobiledevice3.remote.common import TunnelProtocol
+from pymobiledevice3.remote.remote_service import RemoteService
+from pymobiledevice3.remote.remote_service_discovery import RemoteServiceDiscoveryService
+from pymobiledevice3.remote.utils import resume_remoted_if_required, stop_remoted_if_required
+from pymobiledevice3.remote.xpc_message import XpcInt64Type, XpcUInt64Type
+from pymobiledevice3.utils import asyncio_print_traceback, set_keepalive
+
+if sys.platform == 'darwin':
+    LOOKBACK_HEADER = struct.pack('>I', AF_INET6)
+else:
+    LOOKBACK_HEADER = b'\x00\x00\x86\xdd'
+
+if sys.platform == 'win32':
+    def wintun_logger(level: int, timestamp: int, message: str) -> None:
+        logging.getLogger('wintun').info(message)
+
+    set_logger(wintun_logger)
+
+IPV6_HEADER_SIZE = 40
+UDP_HEADER_SIZE = 8
+
+# The iOS device uses an MTU of 1500, so we'll have to increase the default QUIC MTU
+IOS_DEVICE_MTU_SIZE = 1500
+packet_builder.PACKET_MAX_SIZE = IOS_DEVICE_MTU_SIZE - IPV6_HEADER_SIZE - UDP_HEADER_SIZE
+
+PairingDataComponentType = ConstructEnum(Int8ul,
+                                         METHOD=0x00,
+                                         IDENTIFIER=0x01,
+                                         SALT=0x02,
+                                         PUBLIC_KEY=0x03,
+                                         PROOF=0x04,
+                                         ENCRYPTED_DATA=0x05,
+                                         STATE=0x06,
+                                         ERROR=0x07,
+                                         RETRY_DELAY=0x08,
+                                         CERTIFICATE=0x09,
+                                         SIGNATURE=0x0a,
+                                         PERMISSIONS=0x0b,
+                                         FRAGMENT_DATA=0x0c,
+                                         FRAGMENT_LAST=0x0d,
+                                         SESSION_ID=0x0e,
+                                         TTL=0x0f,
+                                         EXTRA_DATA=0x10,
+                                         INFO=0x11,
+                                         ACL=0x12,
+                                         FLAGS=0x13,
+                                         VALIDATION_DATA=0x14,
+                                         MFI_AUTH_TOKEN=0x15,
+                                         MFI_PRODUCT_TYPE=0x16,
+                                         SERIAL_NUMBER=0x17,
+                                         MFI_AUTH_TOKEN_UUID=0x18,
+                                         APP_FLAGS=0x19,
+                                         OWNERSHIP_PROOF=0x1a,
+                                         SETUP_CODE_TYPE=0x1b,
+                                         PRODUCTION_DATA=0x1c,
+                                         APP_INFO=0x1d,
+                                         SEPARATOR=0xff)
 
 PairingDataComponentTLV8 = Struct(
     'type' / PairingDataComponentType,
@@ -82,36 +135,178 @@ CDTunnelPacket = Struct(
 )
 
 
-class RemotePairingTunnel(QuicConnectionProtocol):
-    MTU = 1420
+class RemotePairingTunnel(ABC):
+    REQUESTED_MTU = 1420
 
-    def __init__(self, quic: QuicConnection, stream_handler: Optional[QuicStreamHandler] = None):
-        super().__init__(quic, stream_handler)
+    def __init__(self):
         self._queue = asyncio.Queue()
+        self._tun_read_task = None
+        self._logger = logging.getLogger(f'{__name__}.{self.__class__.__name__}')
+        self.tun = None
 
+    @abstractmethod
+    async def send_packet_to_device(self, packet: bytes) -> None:
+        pass
+
+    @abstractmethod
     async def request_tunnel_establish(self) -> Mapping:
-        stream_id = self._quic.get_next_available_stream_id()
-        # TODO: understand what this buffer should actually do
-        self._quic.send_datagram_frame(b'x' * 1024)
-        self._quic.send_stream_data(stream_id, self._encode_cdtunnel_packet(
-            {'type': 'clientHandshakeRequest', 'mtu': self.MTU}))
-        self.transmit()
-        return await self._queue.get()
+        pass
 
-    def quic_event_received(self, event: QuicEvent) -> None:
-        if isinstance(event, StreamDataReceived):
-            self._queue.put_nowait(json.loads(CDTunnelPacket.parse(event.data).body))
+    @abstractmethod
+    async def wait_closed(self) -> None:
+        pass
+
+    @asyncio_print_traceback
+    async def tun_read_task(self) -> None:
+        read_size = self.tun.mtu + len(LOOKBACK_HEADER)
+        if sys.platform != 'win32':
+            async with aiofiles.open(self.tun.fileno(), 'rb', opener=lambda path, flags: path, buffering=0) as f:
+                while True:
+                    packet = await f.read(read_size)
+                    assert packet.startswith(LOOKBACK_HEADER)
+                    packet = packet[len(LOOKBACK_HEADER):]
+                    await self.send_packet_to_device(packet)
+        else:
+            while True:
+                packet = await asyncio.get_running_loop().run_in_executor(None, self.tun.read)
+                if packet:
+                    await self.send_packet_to_device(packet)
+
+    def start_tunnel(self, address: str, mtu: int) -> None:
+        self.tun = TunTapDevice()
+        self.tun.mtu = mtu
+        self.tun.addr = address
+        self.tun.up()
+        self._tun_read_task = asyncio.create_task(self.tun_read_task())
+
+    async def stop_tunnel(self) -> None:
+        self._logger.debug('stopping tunnel')
+        self._tun_read_task.cancel()
+        with suppress(CancelledError):
+            await self._tun_read_task
+        if sys.platform != 'darwin':
+            self.tun.down()
+        self.tun = None
 
     @staticmethod
     def _encode_cdtunnel_packet(data: Mapping) -> bytes:
         return CDTunnelPacket.build({'body': json.dumps(data).encode()})
 
 
-class CoreDeviceTunnelService:
+class RemotePairingQuicTunnel(RemotePairingTunnel, QuicConnectionProtocol):
+    MAX_QUIC_DATAGRAM = 14000
+    MAX_IDLE_TIMEOUT = 30.0
+    REQUESTED_MTU = 1420
+
+    def __init__(self, quic: QuicConnection, stream_handler: Optional[QuicStreamHandler] = None):
+        RemotePairingTunnel.__init__(self)
+        QuicConnectionProtocol.__init__(self, quic, stream_handler)
+        self._keep_alive_task = None
+
+    async def wait_closed(self) -> None:
+        await QuicConnectionProtocol.wait_closed(self)
+
+    async def send_packet_to_device(self, packet: bytes) -> None:
+        self._quic.send_datagram_frame(packet)
+        self.transmit()
+
+    async def request_tunnel_establish(self) -> Mapping:
+        stream_id = self._quic.get_next_available_stream_id()
+        # pad the data with random data to force the MTU size correctly
+        self._quic.send_datagram_frame(b'x' * 1024)
+        self._quic.send_stream_data(stream_id, self._encode_cdtunnel_packet(
+            {'type': 'clientHandshakeRequest', 'mtu': self.REQUESTED_MTU}))
+        self.transmit()
+        return await self._queue.get()
+
+    @asyncio_print_traceback
+    async def keep_alive_task(self) -> None:
+        while True:
+            await self.ping()
+            await asyncio.sleep(self._quic.configuration.idle_timeout / 2)
+
+    def start_tunnel(self, address: str, mtu: int) -> None:
+        super().start_tunnel(address, mtu)
+        self._keep_alive_task = asyncio.create_task(self.keep_alive_task())
+
+    async def stop_tunnel(self) -> None:
+        self._keep_alive_task.cancel()
+        with suppress(CancelledError):
+            await self._keep_alive_task
+        await super().stop_tunnel()
+
+    def quic_event_received(self, event: QuicEvent) -> None:
+        if isinstance(event, ConnectionTerminated):
+            self.close()
+        elif isinstance(event, StreamDataReceived):
+            self._queue.put_nowait(json.loads(CDTunnelPacket.parse(event.data).body))
+        elif isinstance(event, DatagramFrameReceived):
+            self.tun.write(LOOKBACK_HEADER + event.data)
+
+    @staticmethod
+    def _encode_cdtunnel_packet(data: Mapping) -> bytes:
+        return CDTunnelPacket.build({'body': json.dumps(data).encode()})
+
+
+class RemotePairingTcpTunnel(RemotePairingTunnel):
+    REQUESTED_MTU = 16000
+
+    def __init__(self, reader: StreamReader, writer: StreamWriter):
+        RemotePairingTunnel.__init__(self)
+        self._reader = reader
+        self._writer = writer
+        self._sock_read_task = None
+
+    async def send_packet_to_device(self, packet: bytes) -> None:
+        self._writer.write(packet)
+        await self._writer.drain()
+
+    @asyncio_print_traceback
+    async def sock_read_task(self) -> None:
+        while True:
+            ipv6_header = await self._reader.readexactly(IPV6_HEADER_SIZE)
+            ipv6_length = struct.unpack('>H', ipv6_header[4:6])[0]
+            ipv6_body = await self._reader.readexactly(ipv6_length)
+            self.tun.write(LOOKBACK_HEADER + ipv6_header + ipv6_body)
+
+    async def wait_closed(self) -> None:
+        await self._writer.wait_closed()
+
+    async def request_tunnel_establish(self) -> Mapping:
+        self._writer.write(self._encode_cdtunnel_packet(
+            {'type': 'clientHandshakeRequest', 'mtu': self.REQUESTED_MTU}))
+        await self._writer.drain()
+        return json.loads(CDTunnelPacket.parse(await self._reader.read(self.REQUESTED_MTU)).body)
+
+    def start_tunnel(self, address: str, mtu: int) -> None:
+        super().start_tunnel(address, mtu)
+        self._sock_read_task = asyncio.create_task(self.sock_read_task())
+
+    async def stop_tunnel(self) -> None:
+        self._sock_read_task.cancel()
+        with suppress(CancelledError):
+            await self._sock_read_task
+        if not self._writer.is_closing():
+            self._writer.close()
+            await self._writer.wait_closed()
+        await super().stop_tunnel()
+
+
+@dataclasses.dataclass
+class TunnelResult:
+    interface: str
+    address: str
+    port: int
+    protocol: TunnelProtocol
+    client: RemotePairingTunnel
+
+
+class CoreDeviceTunnelService(RemoteService):
+    SERVICE_NAME = 'com.apple.internal.dt.coredevice.untrusted.tunnelservice'
     WIRE_PROTOCOL_VERSION = 19
 
     def __init__(self, rsd: RemoteServiceDiscoveryService):
-        self._logger = logging.getLogger(__name__)
+        super().__init__(rsd, self.SERVICE_NAME)
         self._sequence_number = 0
         self._encrypted_sequence_number = 0
         self.rsd = rsd
@@ -126,7 +321,7 @@ class CoreDeviceTunnelService:
         self.signature = None
 
     def connect(self, autopair: bool = True) -> None:
-        self.service = self.rsd.connect_to_service('com.apple.internal.dt.coredevice.untrusted.tunnelservice')
+        super().connect()
         self.version = self.service.receive_response()['ServiceVersion']
 
         self._attempt_pair_verify()
@@ -135,45 +330,101 @@ class CoreDeviceTunnelService:
                 self._pair()
         self._init_client_server_main_encryption_keys()
 
-    def create_listener(self, private_key: RSAPrivateKey, protocol: str = 'quic') -> Mapping:
+    def create_quic_listener(self, private_key: RSAPrivateKey) -> Mapping:
         request = {'request': {'_0': {'createListener': {
             'key': base64.b64encode(
                 private_key.public_key().public_bytes(Encoding.DER, PublicFormat.SubjectPublicKeyInfo)
             ).decode(),
-            'transportProtocolType': protocol}}}}
+            'transportProtocolType': 'quic'}}}}
 
         response = self._send_receive_encrypted_request(request)
         return response['createListener']
 
-    async def start_quic_tunnel(self, private_key: RSAPrivateKey, secrets_log_file: Optional[TextIO] = None) -> Mapping:
-        parameters = self.create_listener(private_key, protocol='quic')
+    def create_tcp_listener(self) -> Mapping:
+        request = {'request': {'_0': {'createListener': {
+            'key': base64.b64encode(
+                self.encryption_key,
+            ).decode(),
+            'transportProtocolType': 'tcp'}}}}
+
+        response = self._send_receive_encrypted_request(request)
+        return response['createListener']
+
+    @asynccontextmanager
+    async def start_quic_tunnel(
+            self, secrets_log_file: Optional[TextIO] = None,
+            max_idle_timeout: float = RemotePairingQuicTunnel.MAX_IDLE_TIMEOUT) -> AsyncGenerator[TunnelResult, None]:
+        private_key = rsa.generate_private_key(public_exponent=65537, key_size=2048)
+        parameters = self.create_quic_listener(private_key)
         cert = make_cert(private_key, private_key.public_key())
-        configuration = QuicConfiguration(alpn_protocols=['RemotePairingTunnelProtocol'],
-                                          is_client=True,
-                                          certificate=cert,
-                                          private_key=private_key,
-                                          verify_mode=VerifyMode.CERT_NONE)
+        configuration = QuicConfiguration(
+            alpn_protocols=['RemotePairingTunnelProtocol'],
+            is_client=True,
+            certificate=cert,
+            private_key=private_key,
+            verify_mode=VerifyMode.CERT_NONE,
+            verify_hostname=False,
+            max_datagram_frame_size=RemotePairingQuicTunnel.MAX_QUIC_DATAGRAM,
+            idle_timeout=max_idle_timeout
+        )
         configuration.secrets_log_file = secrets_log_file
 
         host = self.service.address[0]
         port = parameters['port']
 
-        self._logger.debug(f'Connecting to {host}:{port}')
+        self.logger.debug(f'Connecting to {host}:{port}')
         async with aioquic_connect(
                 host,
                 port,
                 configuration=configuration,
-                create_protocol=RemotePairingTunnel,
+                create_protocol=RemotePairingQuicTunnel,
         ) as client:
-            self._logger.debug('quic connected')
-            return await client.request_tunnel_establish()
+            self.logger.debug('quic connected')
+            client = cast(RemotePairingQuicTunnel, client)
+            await client.wait_connected()
+            handshake_response = await client.request_tunnel_establish()
+            client.start_tunnel(handshake_response['clientParameters']['address'],
+                                handshake_response['clientParameters']['mtu'])
+            try:
+                yield TunnelResult(
+                    client.tun.name, handshake_response['serverAddress'], handshake_response['serverRSDPort'],
+                    TunnelProtocol.QUIC, client)
+            finally:
+                await client.stop_tunnel()
+
+    @asynccontextmanager
+    async def start_tcp_tunnel(self) -> AsyncGenerator[TunnelResult, None]:
+        parameters = self.create_tcp_listener()
+        host = self.service.address[0]
+        port = parameters['port']
+        sock = create_connection((host, port))
+        set_keepalive(sock)
+        ctx = SSLPSKContext(ssl.PROTOCOL_TLSv1_2)
+        ctx.psk = self.encryption_key
+        ctx.set_ciphers('PSK')
+        reader, writer = await asyncio.open_connection(sock=sock, ssl=ctx, server_hostname='')
+        tunnel = RemotePairingTcpTunnel(reader, writer)
+        handshake_response = await tunnel.request_tunnel_establish()
+
+        tunnel.start_tunnel(handshake_response['clientParameters']['address'],
+                            handshake_response['clientParameters']['mtu'])
+
+        try:
+            yield TunnelResult(
+                tunnel.tun.name, handshake_response['serverAddress'], handshake_response['serverRSDPort'],
+                TunnelProtocol.TCP, tunnel)
+        finally:
+            await tunnel.stop_tunnel()
 
     def save_pair_record(self) -> None:
         self.pair_record_path.write_bytes(
             plistlib.dumps({
                 'public_key': self.ed25519_private_key.public_key().public_bytes_raw(),
                 'private_key': self.ed25519_private_key.private_bytes_raw(),
+                'remote_unlock_host_key': self.remote_unlock_host_key
             }))
+        if getenv('SUDO_UID') and sys.platform != 'win32':
+            chown(self.pair_record_path, int(getenv('SUDO_UID')), int(getenv('SUDO_GID')))
 
     @property
     def pair_record(self) -> Optional[Mapping]:
@@ -184,7 +435,8 @@ class CoreDeviceTunnelService:
     @property
     def pair_record_path(self) -> Path:
         pair_records_cache_directory = create_pairing_records_cache_folder()
-        return pair_records_cache_directory / f'remote_{self.handshake_info["peerDeviceInfo"]["identifier"]}.plist'
+        identifier = self.handshake_info['peerDeviceInfo']['identifier']
+        return pair_records_cache_directory / f'{get_remote_pairing_record_filename(identifier)}.{PAIRING_RECORD_EXT}'
 
     def _pair(self) -> None:
         pairing_consent_result = self._request_pair_consent()
@@ -207,10 +459,16 @@ class CoreDeviceTunnelService:
                                  'kind': 'setupManualPairing',
                                  'sendingHost': platform.node(),
                                  'startNewSession': True})
-        assert 'awaitingUserConsent' in self._receive_plain_response()['event']['_0']
-        response = self._receive_pairing_data()
+        self.logger.info('Waiting user pairing consent')
+        response = self._receive_plain_response()['event']['_0']
+        if 'awaitingUserConsent' in response:
+            pairingData = self._receive_pairing_data()
+        else:
+            # On tvOS no consent is needed and pairing data is returned immediately.
+            pairingData = response['pairingData']['_0']['data']
+
         data = self.decode_tlv(PairingDataComponentTLVBuf.parse(
-            response))
+            pairingData))
         return PairConsentResult(public_key=data[PairingDataComponentType.PUBLIC_KEY],
                                  salt=data[PairingDataComponentType.SALT])
 
@@ -327,7 +585,10 @@ class CoreDeviceTunnelService:
 
     def _create_remote_unlock(self) -> None:
         response = self._send_receive_encrypted_request({'request': {'_0': {'createRemoteUnlockKey': {}}}})
-        self.remote_unlock_host_key = response['createRemoteUnlockKey']['hostKey']
+        if 'errorExtended' in response:
+            self.remote_unlock_host_key = None
+        else:
+            self.remote_unlock_host_key = response['createRemoteUnlockKey']['hostKey']
 
     def _attempt_pair_verify(self) -> None:
         self.handshake_info = self._send_receive_handshake({
@@ -345,6 +606,11 @@ class CoreDeviceTunnelService:
                                                     'startNewSession': True})
 
         data = self.decode_tlv(PairingDataComponentTLVBuf.parse(response))
+
+        if PairingDataComponentType.ERROR in data:
+            self._send_pair_verify_failed()
+            return False
+
         peer_public_key = X25519PublicKey.from_public_bytes(data[PairingDataComponentType.PUBLIC_KEY])
         self.encryption_key = self.x25519_private_key.exchange(peer_public_key)
 
@@ -428,7 +694,15 @@ class CoreDeviceTunnelService:
         self._send_plain_request({'event': {'_0': {'pairingData': {'_0': pairing_data}}}})
 
     def _receive_pairing_data(self) -> Mapping:
-        return self._receive_plain_response()['event']['_0']['pairingData']['_0']['data']
+        response = self._receive_plain_response()['event']['_0']
+        if 'pairingData' in response:
+            return response['pairingData']['_0']['data']
+        if 'pairingRejectedWithError' in response:
+            raise UserDeniedPairingError(response['pairingRejectedWithError']
+                                         .get('wrappedError', {})
+                                         .get('userInfo', {})
+                                         .get('NSLocalizedDescription'))
+        raise PyMobileDevice3Exception(f'Got an unknown state message: {response}')
 
     def _send_receive_plain_request(self, plain_request: Mapping):
         self._send_plain_request(plain_request)
@@ -467,3 +741,33 @@ def create_core_device_tunnel_service(rsd: RemoteServiceDiscoveryService, autopa
     service = CoreDeviceTunnelService(rsd)
     service.connect(autopair=autopair)
     return service
+
+
+@asynccontextmanager
+async def start_tunnel(service_provider: RemoteServiceDiscoveryService, secrets: Optional[TextIO] = None,
+                       max_idle_timeout: float = RemotePairingQuicTunnel.MAX_IDLE_TIMEOUT,
+                       protocol: TunnelProtocol = TunnelProtocol.QUIC) \
+        -> AsyncGenerator[TunnelResult, None]:
+    stop_remoted_if_required()
+    with create_core_device_tunnel_service(service_provider, autopair=True) as service:
+        if protocol == TunnelProtocol.QUIC:
+            async with service.start_quic_tunnel(
+                    secrets_log_file=secrets, max_idle_timeout=max_idle_timeout) as tunnel_result:
+                resume_remoted_if_required()
+                yield tunnel_result
+        elif protocol == TunnelProtocol.TCP:
+            async with service.start_tcp_tunnel() as tunnel_result:
+                resume_remoted_if_required()
+                yield tunnel_result
+
+
+async def start_quic_tunnel(service_provider: RemoteServiceDiscoveryService, secrets: Optional[TextIO] = None,
+                            max_idle_timeout: float = RemotePairingQuicTunnel.MAX_IDLE_TIMEOUT) \
+        -> AsyncGenerator[TunnelResult, None]:
+    return start_tunnel(service_provider, secrets, max_idle_timeout, protocol=TunnelProtocol.QUIC)
+
+
+async def start_tcp_tunnel(service_provider: RemoteServiceDiscoveryService, secrets: Optional[TextIO] = None,
+                           max_idle_timeout: float = RemotePairingQuicTunnel.MAX_IDLE_TIMEOUT) \
+        -> AsyncGenerator[TunnelResult, None]:
+    return start_tunnel(service_provider, secrets, max_idle_timeout, protocol=TunnelProtocol.TCP)
