@@ -5,7 +5,6 @@ import socket
 import ssl
 import struct
 import time
-from enum import Enum, auto
 from typing import Mapping, Optional
 
 import IPython
@@ -14,6 +13,11 @@ from pygments import formatters, highlight, lexers
 from pymobiledevice3.exceptions import ConnectionFailedError, ConnectionTerminatedError, NoDeviceConnectedError, \
     PyMobileDevice3Exception
 from pymobiledevice3.usbmux import MuxDevice, select_device
+from pymobiledevice3.utils import set_keepalive
+
+DEFAULT_AFTER_IDLE_SEC = 3
+DEFAULT_INTERVAL_SEC = 3
+DEFAULT_MAX_FAILS = 3
 
 SHELL_USAGE = """
 # This shell allows you to communicate directly with every service layer behind the lockdownd daemon.
@@ -57,12 +61,7 @@ def create_context(certfile, keyfile=None):
     return context
 
 
-class Medium(Enum):
-    TCP = auto()
-    USBMUX = auto()
-
-
-class ServiceConnection:
+class LockdownServiceConnection:
     """ wrapper for usbmux tcp-relay connections """
 
     def __init__(self, sock: socket.socket, mux_device: MuxDevice = None):
@@ -76,26 +75,22 @@ class ServiceConnection:
         self._writer = None  # type: Optional[asyncio.StreamWriter]
 
     @staticmethod
-    def create_using_tcp(hostname: str, port: int) -> 'ServiceConnection':
+    def create_using_tcp(hostname: str, port: int, keep_alive: bool = True) -> 'LockdownServiceConnection':
         sock = socket.create_connection((hostname, port))
-        return ServiceConnection(sock)
+        if keep_alive:
+            set_keepalive(sock)
+        return LockdownServiceConnection(sock)
 
     @staticmethod
-    def create_using_usbmux(udid: Optional[str], port: int, connection_type: str = None) -> 'ServiceConnection':
-        target_device = select_device(udid, connection_type=connection_type)
+    def create_using_usbmux(udid: Optional[str], port: int, connection_type: str = None,
+                            usbmux_address: Optional[str] = None) -> 'LockdownServiceConnection':
+        target_device = select_device(udid, connection_type=connection_type, usbmux_address=usbmux_address)
         if target_device is None:
             if udid:
                 raise ConnectionFailedError()
             raise NoDeviceConnectedError()
-        sock = target_device.connect(port)
-        return ServiceConnection(sock, mux_device=target_device)
-
-    @staticmethod
-    def create(medium: Medium, identifier: str, port: int, connection_type: str = None) -> 'ServiceConnection':
-        if medium == Medium.TCP:
-            return ServiceConnection.create_using_tcp(identifier, port)
-        else:
-            return ServiceConnection.create_using_usbmux(identifier, port, connection_type=connection_type)
+        sock = target_device.connect(port, usbmux_address=usbmux_address)
+        return LockdownServiceConnection(sock, mux_device=target_device)
 
     def setblocking(self, blocking: bool) -> None:
         self.socket.setblocking(blocking)
@@ -187,6 +182,9 @@ class ServiceConnection:
             server_hostname=''
         )
 
+    async def aio_start(self) -> None:
+        self._reader, self._writer = await asyncio.open_connection(sock=self.socket)
+
     def shell(self) -> None:
         IPython.embed(
             header=highlight(SHELL_USAGE, lexers.PythonLexer(), formatters.TerminalTrueColorFormatter(style='native')),
