@@ -1,3 +1,4 @@
+import json
 import typing
 from dataclasses import dataclass
 from enum import Enum
@@ -5,6 +6,7 @@ from enum import Enum
 from packaging.version import Version
 
 from pymobiledevice3.lockdown import LockdownClient
+from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
 from pymobiledevice3.services.remote_server import MessageAux, RemoteServer
 
 
@@ -72,12 +74,64 @@ class AXAuditDeviceSetting_v1(SerializedObject):
         return f'<AXAuditDeviceSetting_v1 {self.key} = {self.value}>'
 
 
+class AXAuditIssue_v1(SerializedObject):
+    FIELDS = ('ElementRectValue_v1', 'IssueClassificationValue_v1',
+              'ElementRectValue_v1', 'FontSizeValue_v1', 'MLGeneratedDescriptionValue_v1')
+
+    def __init__(self, fields):
+        super().__init__(fields)
+
+        for k in self.FIELDS:
+            if k not in self._fields:
+                self._fields[k] = None
+
+    @property
+    def rect(self) -> str:
+        return self._fields['ElementRectValue_v1']
+
+    @property
+    def issue_type(self) -> typing.Any:
+        if self._fields['IssueClassificationValue_v1'] in AUDIT_TYPES:
+            return AUDIT_TYPES[self._fields['IssueClassificationValue_v1']]
+        else:
+            return self._fields['IssueClassificationValue_v1']
+
+    @property
+    def ml_generated_description(self) -> typing.Any:
+        return self._fields['MLGeneratedDescriptionValue_v1']
+
+    @property
+    def font_size(self) -> typing.Any:
+        return self._fields['FontSizeValue_v1']
+
+    def json(self) -> typing.Mapping:
+        resp = {}
+        resp['element_rect_value'] = self.rect
+        resp['issue_classification'] = self.issue_type
+        resp['font_size'] = self.font_size
+        resp['ml_generated_description'] = self.ml_generated_description
+        return resp
+
+    def __str__(self) -> str:
+        return json.dumps(self.json())
+
+
 SERIALIZABLE_OBJECTS = {
     'AXAuditDeviceSetting_v1': AXAuditDeviceSetting_v1,
     'AXAuditInspectorFocus_v1': AXAuditInspectorFocus_v1,
     'AXAuditElement_v1': AXAuditElement_v1,
     'AXAuditInspectorSection_v1': AXAuditInspectorSection_v1,
     'AXAuditElementAttribute_v1': AXAuditElementAttribute_v1,
+    'AXAuditIssue_v1': AXAuditIssue_v1
+}
+
+AUDIT_TYPES = {
+    3001: 'testTypeDynamicText',
+    3003: 'testTypeTextClipped',
+    1000: 'testTypeElementDetection',
+    5000: 'testTypeSufficientElementDescription',
+    100: 'testTypeHitRegion',
+    13: 'testTypeContrast'
 }
 
 
@@ -115,9 +169,13 @@ def deserialize_object(d):
 
 class AccessibilityAudit(RemoteServer):
     SERVICE_NAME = 'com.apple.accessibility.axAuditDaemon.remoteserver'
+    RSD_SERVICE_NAME = 'com.apple.accessibility.axAuditDaemon.remoteserver.shim.remote'
 
-    def __init__(self, lockdown: LockdownClient):
-        super().__init__(lockdown, self.SERVICE_NAME, remove_ssl_context=True)
+    def __init__(self, lockdown: LockdownServiceProvider):
+        if isinstance(lockdown, LockdownClient):
+            super().__init__(lockdown, self.SERVICE_NAME, remove_ssl_context=True, is_developer_service=False)
+        else:
+            super().__init__(lockdown, self.RSD_SERVICE_NAME, remove_ssl_context=True, is_developer_service=False)
 
         # flush previously received messages
         self.recv_plist()
@@ -129,6 +187,19 @@ class AccessibilityAudit(RemoteServer):
         self.broadcast.deviceCapabilities()
         return self.recv_plist()[0]
 
+    def run_audit(self, value: typing.List) -> typing.List[AXAuditIssue_v1]:
+        self.broadcast.deviceBeginAuditTypes_(MessageAux().append_obj(value))
+
+        while True:
+            message = self.recv_plist()
+            if message[1] is None or message[0] != 'hostDeviceDidCompleteAuditCategoriesWithAuditIssues:':
+                continue
+            return deserialize_object(message[1])[0]['value']
+
+    def supported_audits_types(self) -> None:
+        self.broadcast.deviceAllSupportedAuditTypes()
+        return deserialize_object(self.recv_plist()[0])
+
     @property
     def settings(self) -> typing.List[AXAuditDeviceSetting_v1]:
         self.broadcast.deviceAccessibilitySettings()
