@@ -2,27 +2,46 @@ import logging
 import posixpath
 from typing import Generator, List
 
-from cmd2 import Cmd2ArgumentParser, with_argparser
 from pycrashreport.crash_report import get_crash_report_from_buf
+from xonsh.built_ins import XSH
+from xonsh.cli_utils import Annotated, Arg
 
 from pymobiledevice3.exceptions import AfcException
 from pymobiledevice3.lockdown import LockdownClient
-from pymobiledevice3.services.afc import AfcService, AfcShell
+from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
+from pymobiledevice3.services.afc import AfcService, AfcShell, path_completer
 from pymobiledevice3.services.os_trace import OsTraceService
 
 SYSDIAGNOSE_PROCESS_NAMES = ('sysdiagnose', 'sysdiagnosed')
+SYSDIAGNOSE_DIR = 'DiagnosticLogs/sysdiagnose'
+SYSDIAGNOSE_IN_PROGRESS_MAX_TTL_SECS = 600
+
+# on iOS17, we need to wait for a moment before tryint to fetch the sysdiagnose archive
+IOS17_SYSDIAGNOSE_DELAY = 1
 
 
 class CrashReportsManager:
     COPY_MOBILE_NAME = 'com.apple.crashreportcopymobile'
+    RSD_COPY_MOBILE_NAME = 'com.apple.crashreportcopymobile.shim.remote'
+
     CRASH_MOVER_NAME = 'com.apple.crashreportmover'
+    RSD_CRASH_MOVER_NAME = 'com.apple.crashreportmover.shim.remote'
+
     APPSTORED_PATH = '/com.apple.appstored'
     IN_PROGRESS_SYSDIAGNOSE_EXTENSIONS = ['.tmp', '.tar.gz']
 
-    def __init__(self, lockdown: LockdownClient):
+    def __init__(self, lockdown: LockdownServiceProvider):
         self.logger = logging.getLogger(__name__)
         self.lockdown = lockdown
-        self.afc = AfcService(lockdown, service_name=self.COPY_MOBILE_NAME)
+
+        if isinstance(lockdown, LockdownClient):
+            self.copy_mobile_service_name = self.COPY_MOBILE_NAME
+            self.crash_mover_service_name = self.CRASH_MOVER_NAME
+        else:
+            self.copy_mobile_service_name = self.RSD_COPY_MOBILE_NAME
+            self.crash_mover_service_name = self.RSD_CRASH_MOVER_NAME
+
+        self.afc = AfcService(lockdown, service_name=self.copy_mobile_service_name)
 
     def __enter__(self):
         return self
@@ -78,7 +97,7 @@ class CrashReportsManager:
     def flush(self) -> None:
         """ Trigger com.apple.crashreportmover to flush all products into CrashReports directory """
         ack = b'ping\x00'
-        assert ack == self.lockdown.start_service(self.CRASH_MOVER_NAME).recvall(len(ack))
+        assert ack == self.lockdown.start_lockdown_service(self.crash_mover_service_name).recvall(len(ack))
 
     def watch(self, name: str = None, raw: bool = False) -> Generator[str, None, None]:
         """
@@ -115,50 +134,61 @@ class CrashReportsManager:
         :param out: filename
         :param erase: remove after pulling
         """
-        sysdiagnose_filename = None
-
-        for syslog_entry in OsTraceService(lockdown=self.lockdown).syslog():
-            if (posixpath.basename(syslog_entry.filename) not in SYSDIAGNOSE_PROCESS_NAMES) or \
-                    (posixpath.basename(syslog_entry.image_name) not in SYSDIAGNOSE_PROCESS_NAMES):
-                # filter only sysdianose lines
-                continue
-
-            message = syslog_entry.message
-
-            if message.startswith('SDArchive: Successfully created tar at '):
-                self.logger.info('sysdiagnose creation has begun')
-                for filename in self.ls('DiagnosticLogs/sysdiagnose'):
-                    # search for an IN_PROGRESS archive
-                    if 'IN_PROGRESS_' in filename:
-                        for ext in self.IN_PROGRESS_SYSDIAGNOSE_EXTENSIONS:
-                            if filename.endswith(ext):
-                                sysdiagnose_filename = filename.rsplit(ext)[0]
-                                sysdiagnose_filename = sysdiagnose_filename.replace('IN_PROGRESS_', '')
-                                sysdiagnose_filename = f'{sysdiagnose_filename}.tar.gz'
-                                break
-                break
-
-        self.afc.wait_exists(sysdiagnose_filename)
+        sysdiagnose_filename = self._get_new_sysdiagnose_filename()
+        self.logger.info('sysdiagnose tarball creation has been started')
+        self._wait_for_sysdiagnose_to_finish()
         self.pull(out, entry=sysdiagnose_filename, erase=erase)
 
+    @staticmethod
+    def _sysdiagnose_complete_syslog_match(message: str) -> bool:
+        return message == 'sysdiagnose (full) complete' or 'Sysdiagnose completed' in message
 
-parse_parser = Cmd2ArgumentParser(description='parse given crash report file')
-parse_parser.add_argument('filename')
+    def _wait_for_sysdiagnose_to_finish(self) -> None:
+        with OsTraceService(self.lockdown) as os_trace:
+            for entry in os_trace.syslog():
+                if CrashReportsManager._sysdiagnose_complete_syslog_match(entry.message):
+                    break
 
-clear_parser = Cmd2ArgumentParser(description='remove all crash reports')
+    def _get_new_sysdiagnose_filename(self) -> str:
+        sysdiagnose_filename = None
+        excluded_temp_files = []
+
+        while sysdiagnose_filename is None:
+            try:
+                for filename in self.afc.listdir(SYSDIAGNOSE_DIR):
+                    # search for an IN_PROGRESS archive
+                    if filename not in excluded_temp_files and 'IN_PROGRESS_' in filename:
+                        for ext in self.IN_PROGRESS_SYSDIAGNOSE_EXTENSIONS:
+                            if filename.endswith(ext):
+                                delta = self.lockdown.date - \
+                                    self.afc.stat(posixpath.join(SYSDIAGNOSE_DIR, filename))['st_mtime']
+                                # Ignores IN_PROGRESS sysdiagnose files older than the defined time to live
+                                if delta.total_seconds() < SYSDIAGNOSE_IN_PROGRESS_MAX_TTL_SECS:
+                                    sysdiagnose_filename = filename.rsplit(ext)[0]
+                                    sysdiagnose_filename = sysdiagnose_filename.replace('IN_PROGRESS_', '')
+                                    sysdiagnose_filename = f'{sysdiagnose_filename}.tar.gz'
+                                    return posixpath.join(SYSDIAGNOSE_DIR,  sysdiagnose_filename)
+                                else:
+                                    self.logger.warning(f"Old sysdiagnose temp file ignored {filename}")
+                                    excluded_temp_files.append(filename)
+            except AfcException:
+                pass
 
 
 class CrashReportsShell(AfcShell):
-    def __init__(self, lockdown: LockdownClient):
-        super().__init__(lockdown, service_name=CrashReportsManager.COPY_MOBILE_NAME)
-        self.manager = CrashReportsManager(lockdown)
-        self.complete_parse = self._complete_first_arg
+    @classmethod
+    def create(cls, service_provider: LockdownServiceProvider, **kwargs):
+        manager = CrashReportsManager(service_provider)
+        XSH.ctx['_manager'] = manager
+        super(CrashReportsShell, CrashReportsShell).create(service_provider, service=manager.afc)
 
-    @with_argparser(parse_parser)
-    def do_parse(self, args) -> None:
-        self.poutput(
-            get_crash_report_from_buf(self.afc.get_file_contents(args.filename).decode(), filename=args.filename))
+    def _setup_shell_commands(self):
+        super()._setup_shell_commands()
+        self._register_arg_parse_alias('parse', self._do_parse)
+        self._register_arg_parse_alias('clear', self._do_clear)
 
-    @with_argparser(clear_parser)
-    def do_clear(self, args) -> None:
-        self.manager.clear()
+    def _do_parse(self, filename: Annotated[str, Arg(completer=path_completer)]) -> None:
+        print(get_crash_report_from_buf(self.afc.get_file_contents(filename).decode(), filename=filename))
+
+    def _do_clear(self) -> None:
+        XSH.ctx['_manager'].clear()
