@@ -2,13 +2,15 @@ import ctypes
 import datetime
 import shutil
 import struct
+import warnings
 from pathlib import Path
 
-from pymobiledevice3.exceptions import PyMobileDevice3Exception
+from pymobiledevice3.exceptions import NotEnoughDiskSpaceError, PyMobileDevice3Exception
 
 SIZE_FORMAT = '>I'
 CODE_FORMAT = '>B'
 CODE_FILE_DATA = 0xc
+CODE_ERROR_REMOTE = 0xb
 CODE_ERROR_LOCAL = 0x6
 CODE_SUCCESS = 0
 FILE_TRANSFER_TERMINATOR = b'\x00\x00\x00\x00'
@@ -38,6 +40,7 @@ class DeviceLink:
             'DLMessageDownloadFiles': self.download_files,
             'DLContentsOfDirectory': self.contents_of_directory,
             'DLMessageCopyItem': self.copy_item,
+            'DLMessagePurgeDiskSpace': self.purge_disk_space
         }
 
     def dl_loop(self, progress_callback=lambda x: None):
@@ -103,10 +106,12 @@ class DeviceLink:
                 ftype = 'DLFileTypeDirectory'
             if file.is_file():
                 ftype = 'DLFileTypeRegular'
+            modifications_data = datetime.datetime.fromtimestamp(file.stat().st_mtime - APPLE_EPOCH)
+            modifications_data = modifications_data.replace(tzinfo=None)
             data[file.name] = {
                 'DLFileType': ftype,
                 'DLFileSize': file.stat().st_size,
-                'DLFileModificationDate': datetime.datetime.utcfromtimestamp(file.stat().st_mtime - APPLE_EPOCH)
+                'DLFileModificationDate': modifications_data
             }
         self.status_response(0, status_dict=data)
 
@@ -125,6 +130,12 @@ class DeviceLink:
                     size, = struct.unpack(SIZE_FORMAT, self.service.recvall(struct.calcsize(SIZE_FORMAT)))
                     code, = struct.unpack(CODE_FORMAT, self.service.recvall(struct.calcsize(CODE_FORMAT)))
                     size -= struct.calcsize(CODE_FORMAT)
+            if code == CODE_ERROR_REMOTE:
+                # iOS 17 beta devices give this error for: backup_manifest.db
+                error_message = self.service.recvall(size).decode()
+                warnings.warn(f'Failed to fully upload: {file_name}. Device file name: {device_name}. Reason: '
+                              f'{error_message}')
+                continue
             assert code == CODE_SUCCESS
         self.status_response(0)
 
@@ -149,6 +160,9 @@ class DeviceLink:
             shutil.copy(src, dest)
         self.status_response(0)
 
+    def purge_disk_space(self, message) -> None:
+        raise NotEnoughDiskSpaceError()
+
     def remove_items(self, message):
         for path in message[1]:
             rm_path = self.root_path / path
