@@ -232,10 +232,11 @@ thread_group_snapshot_trace_v2 = Struct(
 
 thread_group_snapshot_trace_v3 = Struct(
     'tgs_id' / Int64ul,
-    '_tgs_name' / FixedSized(16, GreedyString('utf8')),
+    '_tgs_name' / Bytes(16),
     'tgs_flags' / Int64ul,
-    '_tgs_name_cont' / FixedSized(16, GreedyString('utf8')),
-    'tgs_name' / Computed(lambda ctx: ctx._tgs_name.strip('\x00') + ctx._tgs_name_cont.strip('\x00')),
+    '_tgs_name_cont' / Bytes(16),
+    'tgs_name' / Computed(
+        lambda ctx: (ctx._tgs_name.strip(b'\x00') + ctx._tgs_name_cont.strip(b'\x00')).decode('utf-8')),
 )
 
 thread_group_snapshot = Struct(
@@ -676,8 +677,11 @@ class CoreProfileSessionTap(Tap):
         mach_absolute_time = time_info[0]
         numer = time_info[1]
         denom = time_info[2]
+
         usecs_since_epoch = dvt.lockdown.get_value(key='TimeIntervalSince1970') * 1000000
+        timezone_ = timezone(timedelta(seconds=dvt.lockdown.get_value(key='TimeZoneOffsetFromUTC')))
+
         return dict(
             numer=numer, denom=denom, mach_absolute_time=mach_absolute_time, usecs_since_epoch=usecs_since_epoch,
-            timezone=timezone(timedelta(seconds=dvt.lockdown.get_value(key='TimeZoneOffsetFromUTC')))
+            timezone=timezone_
         )
