@@ -8,19 +8,31 @@ from packaging.version import Version
 
 from pymobiledevice3.common import get_home_folder
 from pymobiledevice3.exceptions import AlreadyMountedError, DeveloperDiskImageNotFoundError, \
-    DeveloperModeIsNotEnabledError, InternalError, MessageNotSupportedError, MissingManifestError, NotMountedError, \
-    PyMobileDevice3Exception, UnsupportedCommandError
+    DeveloperModeIsNotEnabledError, InternalError, MessageNotSupportedError, MissingManifestError, \
+    NoSuchBuildIdentityError, NotMountedError, PyMobileDevice3Exception, UnsupportedCommandError
 from pymobiledevice3.lockdown import LockdownClient
+from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
 from pymobiledevice3.restore.tss import TSSRequest
-from pymobiledevice3.services.base_service import BaseService
+from pymobiledevice3.services.lockdown_service import LockdownService
 
 
-class MobileImageMounterService(BaseService):
+class MobileImageMounterService(LockdownService):
     # implemented in /usr/libexec/mobile_storage_proxy
     SERVICE_NAME = 'com.apple.mobile.mobile_image_mounter'
+    RSD_SERVICE_NAME = 'com.apple.mobile.mobile_image_mounter.shim.remote'
+    IMAGE_TYPE: str = None
 
-    def __init__(self, lockdown: LockdownClient):
-        super().__init__(lockdown, self.SERVICE_NAME)
+    def __init__(self, lockdown: LockdownServiceProvider):
+        if isinstance(lockdown, LockdownClient):
+            super().__init__(lockdown, self.SERVICE_NAME)
+        else:
+            super().__init__(lockdown, self.RSD_SERVICE_NAME)
+
+    def raise_if_cannot_mount(self) -> None:
+        if self.is_image_mounted(self.IMAGE_TYPE):
+            raise AlreadyMountedError()
+        if Version(self.lockdown.product_version).major >= 16 and not self.lockdown.developer_mode_status:
+            raise DeveloperModeIsNotEnabledError()
 
     def copy_devices(self) -> List[Mapping]:
         """ Copy mounted devices list. """
@@ -168,8 +180,7 @@ class DeveloperDiskImageMounter(MobileImageMounterService):
     IMAGE_TYPE = 'Developer'
 
     def mount(self, image: Path, signature: Path) -> None:
-        if self.is_image_mounted(self.IMAGE_TYPE):
-            raise AlreadyMountedError()
+        self.raise_if_cannot_mount()
 
         image = Path(image).read_bytes()
         signature = Path(signature).read_bytes()
@@ -185,8 +196,7 @@ class PersonalizedImageMounter(MobileImageMounterService):
 
     def mount(self, image: Path, build_manifest: Path, trust_cache: Path,
               info_plist: Mapping = None) -> None:
-        if self.is_image_mounted(self.IMAGE_TYPE):
-            raise AlreadyMountedError()
+        self.raise_if_cannot_mount()
 
         image = image.read_bytes()
         trust_cache = trust_cache.read_bytes()
@@ -197,7 +207,7 @@ class PersonalizedImageMounter(MobileImageMounterService):
         try:
             manifest = self.query_personalization_manifest('DeveloperDiskImage', hashlib.sha384(image).digest())
         except MissingManifestError:
-            self.service = self.lockdown.start_service(self.SERVICE_NAME)
+            self.service = self.lockdown.start_lockdown_service(self.service_name)
             manifest = self.get_manifest_from_tss(plistlib.loads(build_manifest.read_bytes()))
 
         self.upload_image(self.IMAGE_TYPE, image, manifest)
@@ -228,6 +238,8 @@ class PersonalizedImageMounter(MobileImageMounterService):
                     int(tmp_build_identity['ApChipID'], 0) == chip_id:
                 build_identity = tmp_build_identity
                 break
+        else:
+            raise NoSuchBuildIdentityError(f'Could not find the manifest for board {board_id} and chip {chip_id}')
         manifest = build_identity['Manifest']
 
         parameters = {
@@ -275,7 +287,7 @@ class PersonalizedImageMounter(MobileImageMounterService):
 
             # Make sure we have a Digest key for Trusted items even if empty
             if manifest_entry.get('Digest') is None:
-                tss_entry['Digest'] = bytes()
+                tss_entry['Digest'] = b''
 
             request.update({key: tss_entry})
 
@@ -283,7 +295,7 @@ class PersonalizedImageMounter(MobileImageMounterService):
         return response['ApImg4Ticket']
 
 
-def auto_mount_developer(lockdown: LockdownClient, xcode: str = None, version: str = None) -> None:
+def auto_mount_developer(lockdown: LockdownServiceProvider, xcode: str = None, version: str = None) -> None:
     """ auto-detect correct DeveloperDiskImage and mount it """
     if xcode is None:
         # avoid "default"-ing this option, because Windows and Linux won't have this path
@@ -297,7 +309,8 @@ def auto_mount_developer(lockdown: LockdownClient, xcode: str = None, version: s
         raise AlreadyMountedError()
 
     if version is None:
-        version = lockdown.sanitized_ios_version
+        version = Version(lockdown.product_version)
+        version = f'{version.major}.{version.minor}'
     image_dir = f'{xcode}/Contents/Developer/Platforms/iPhoneOS.platform/DeviceSupport/{version}'
     image_path = f'{image_dir}/DeveloperDiskImage.dmg'
     signature = f'{image_path}.signature'
@@ -322,7 +335,7 @@ def auto_mount_developer(lockdown: LockdownClient, xcode: str = None, version: s
     image_mounter.mount(image_path, signature)
 
 
-def auto_mount_personalized(lockdown: LockdownClient) -> None:
+def auto_mount_personalized(lockdown: LockdownServiceProvider) -> None:
     local_path = get_home_folder() / 'Xcode_iOS_DDI_Personalized'
     local_path.mkdir(parents=True, exist_ok=True)
 
@@ -342,7 +355,7 @@ def auto_mount_personalized(lockdown: LockdownClient) -> None:
     PersonalizedImageMounter(lockdown=lockdown).mount(image, build_manifest, trustcache)
 
 
-def auto_mount(lockdown: LockdownClient, xcode: str = None, version: str = None) -> None:
+def auto_mount(lockdown: LockdownServiceProvider, xcode: str = None, version: str = None) -> None:
     if Version(lockdown.product_version) < Version('17.0'):
         auto_mount_developer(lockdown, xcode=xcode, version=version)
     else:
