@@ -10,9 +10,9 @@ from pymobiledevice3.exceptions import AfcException, AfcFileNotFoundError, Conne
     PyMobileDevice3Exception
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.services.afc import AFC_LOCK_EX, AFC_LOCK_UN, AfcService, afc_error_t
-from pymobiledevice3.services.base_service import BaseService
 from pymobiledevice3.services.device_link import DeviceLink
 from pymobiledevice3.services.installation_proxy import InstallationProxyService
+from pymobiledevice3.services.lockdown_service import LockdownService
 from pymobiledevice3.services.notification_proxy import NotificationProxyService
 from pymobiledevice3.services.springboard import SpringBoardServicesService
 
@@ -28,11 +28,15 @@ NP_SYNC_LOCK_REQUEST = 'com.apple.itunes-mobdev.syncLockRequest'
 NP_SYNC_DID_FINISH = 'com.apple.itunes-mobdev.syncDidFinish'
 
 
-class Mobilebackup2Service(BaseService):
+class Mobilebackup2Service(LockdownService):
     SERVICE_NAME = 'com.apple.mobilebackup2'
+    RSD_SERVICE_NAME = 'com.apple.mobilebackup2.shim.remote'
 
     def __init__(self, lockdown: LockdownClient):
-        super().__init__(lockdown, self.SERVICE_NAME)
+        if isinstance(lockdown, LockdownClient):
+            super().__init__(lockdown, self.SERVICE_NAME, include_escrow_bag=True)
+        else:
+            super().__init__(lockdown, self.RSD_SERVICE_NAME, include_escrow_bag=True)
 
     @property
     def will_encrypt(self):
@@ -64,11 +68,13 @@ class Mobilebackup2Service(BaseService):
 
                 # Initialize Status.plist file if doesn't exist.
                 status_path = device_directory / 'Status.plist'
+                current_date = datetime.now()
+                current_date = current_date.replace(tzinfo=None)
                 if full or not status_path.exists():
                     with open(device_directory / 'Status.plist', 'wb') as fd:
                         plistlib.dump({
                             'BackupState': 'new',
-                            'Date': datetime.utcnow(),
+                            'Date': current_date,
                             'IsFullBackup': full,
                             'Version': '3.3',
                             'SnapshotState': 'finished',
